/*******************************************************************************
 *
 *   persistent_region.h
 *   Copyright 2016-2019 NetApp, Inc. All Rights Reserved.
 *
 *   Provides definitions needed to access the persistent memory
 *   managed by eki.
 *
 *******************************************************************************/

#ifndef __INCpersistent_region
#define __INCpersistent_region


/***  INCLUDES  ***/
#include <linux/types.h>

#include "eseries_kernel_interface.h"

/***  CONSTANT DEFINITIONS  ***/
#define REGION_SIGNATURE       "PERSISTENT REGION"
#define REGION_VERSION         0  /* Should increment when region layout changes */
#define ERROR_REGION_VERSION   4
#define ERROR_REGION_SIGNATURE "ERROR REGION"

/* persistent region header and error logging region versions and signatures */
extern uint32_t  regionVersion;
extern uint32_t  errorRegionVersion;
extern char      regionSignature[PR_SIGNATURE_LEN];
extern char      errorRegionSignature[PR_SIGNATURE_LEN];

extern void*           persistent_region_addr;
extern unsigned long   persistent_region_phys_addr;
extern unsigned long   persistent_region_len;

struct region_header
{
    char      signature[PR_SIGNATURE_LEN];
    uint32_t  version;
    uint8_t   padding[4060];   /* Reserve a 4K page for the header */
};

struct error_region
{
    char      signature[PR_SIGNATURE_LEN];
    uint32_t  version;
    uint8_t   logging[ERROR_LOGGING_SIZE];
};

/* If this structure is updated and disrupts existing regions,
 * increment the version in the region header and the regions
 * will not be persisted.
 */
struct persistent_region
{
    struct region_header header;
    struct error_region  err_logging;
};

extern int init_persistent_region(void *kAddr, unsigned long kLen,
                                               unsigned long kPhysAddr);

extern int eki_per_reg_imt(unsigned long arg);

#endif        /* End of __INCpersistent_region */

